@@ -12,6 +12,7 @@ uint64_t get_cas_id(void);
 /*@null@*/
 item *do_item_alloc(char *key, const size_t nkey, const unsigned int flags, const rel_time_t exptime, const int nbytes);
 item_chunk *do_item_alloc_chunk(item_chunk *ch, const size_t bytes_remain);
+item *do_item_alloc_pull(const size_t ntotal, const unsigned int id);
 void item_free(item *it);
 bool item_size_ok(const size_t nkey, const int flags, const int nbytes);
 
@@ -31,6 +32,19 @@ item *do_item_crawl_q(item *it);
 
 void *item_lru_bump_buf_create(void);
 
+#define LRU_PULL_EVICT 1
+#define LRU_PULL_CRAWL_BLOCKS 2
+#define LRU_PULL_RETURN_ITEM 4 /* fill info struct if available */
+
+struct lru_pull_tail_return {
+    item *it;
+    uint32_t hv;
+};
+
+int lru_pull_tail(const int orig_id, const int cur_lru,
+        const uint64_t total_bytes, const uint8_t flags, const rel_time_t max_age,
+        struct lru_pull_tail_return *ret_it);
+
 /*@null@*/
 char *item_cachedump(const unsigned int slabs_clsid, const unsigned int limit, unsigned int *bytes);
 void item_stats(ADD_STAT add_stats, void *c);
@@ -59,7 +73,7 @@ item *do_item_touch(const char *key, const size_t nkey, uint32_t exptime, const
 void item_stats_reset(void);
 extern pthread_mutex_t lru_locks[POWER_LARGEST];
 
-int start_lru_maintainer_thread(void);
+int start_lru_maintainer_thread(void *arg);
 int stop_lru_maintainer_thread(void);
 int init_lru_maintainer(void);
 void lru_maintainer_pause(void);
